@@ -23,9 +23,9 @@
 #include "filter/f-inst.h"
 
 pool *proto_pool;
-list  proto_list;
+list STATIC_LIST_INIT(proto_list);
 
-static list protocol_list;
+static list STATIC_LIST_INIT(protocol_list);
 struct protocol *class_to_protocol[PROTOCOL__MAX];
 
 #define CD(c, msg, args...) ({ if (c->debug & D_STATES) log(L_TRACE "%s.%s: " msg, c->proto->name, c->name ?: "?", ## args); })
@@ -518,11 +518,12 @@ void
 channel_setup_in_table(struct channel *c)
 {
   struct rtable_config *cf = mb_allocz(c->proto->pool, sizeof(struct rtable_config));
+
   cf->name = "import";
   cf->addr_type = c->net_type;
+  cf->internal = 1;
 
-  c->in_table = mb_allocz(c->proto->pool, sizeof(struct rtable));
-  rt_setup(c->proto->pool, c->in_table, cf);
+  c->in_table = rt_setup(c->proto->pool, cf);
 
   c->reload_event = ev_new_init(c->proto->pool, channel_reload_loop, c);
 }
@@ -534,9 +535,9 @@ channel_setup_out_table(struct channel *c)
   struct rtable_config *cf = mb_allocz(c->proto->pool, sizeof(struct rtable_config));
   cf->name = "export";
   cf->addr_type = c->net_type;
+  cf->internal = 1;
 
-  c->out_table = mb_allocz(c->proto->pool, sizeof(struct rtable));
-  rt_setup(c->proto->pool, c->out_table, cf);
+  c->out_table = rt_setup(c->proto->pool, cf);
 }
 
 
@@ -609,6 +610,8 @@ channel_do_down(struct channel *c)
   c->reload_event = NULL;
   c->out_table = NULL;
 
+  /* The in_table and out_table are going to be freed by freeing their resource pools. */
+
   CALL(c->channel->cleanup, c);
 
   /* Schedule protocol shutddown */
@@ -801,6 +804,7 @@ channel_config_get(const struct channel_class *cc, const char *name, uint net_ty
 	cf_error("Multiple %s channels", name);
 
       cf->parent = proto;
+      cf->copy = 1;
       return cf;
     }
 
@@ -826,6 +830,9 @@ static int reconfigure_type;  /* Hack to propagate type info to channel_reconfig
 int
 channel_reconfigure(struct channel *c, struct channel_config *cf)
 {
+  /* Touched by reconfiguration */
+  c->stale = 0;
+
   /* FIXME: better handle these changes, also handle in_keep_filtered */
   if ((c->table != cf->table->table) || (cf->ra_mode && (c->ra_mode != cf->ra_mode)))
     return 0;
@@ -1648,6 +1655,8 @@ proto_build(struct protocol *p)
 /* FIXME: convert this call to some protocol hook */
 extern void bfd_init_all(void);
 
+void protos_build_gen(void);
+
 /**
  * protos_build - build a protocol list
  *
@@ -1660,44 +1669,7 @@ extern void bfd_init_all(void);
 void
 protos_build(void)
 {
-  init_list(&proto_list);
-  init_list(&protocol_list);
-
-  proto_build(&proto_device);
-#ifdef CONFIG_RADV
-  proto_build(&proto_radv);
-#endif
-#ifdef CONFIG_RIP
-  proto_build(&proto_rip);
-#endif
-#ifdef CONFIG_STATIC
-  proto_build(&proto_static);
-#endif
-#ifdef CONFIG_MRT
-  proto_build(&proto_mrt);
-#endif
-#ifdef CONFIG_OSPF
-  proto_build(&proto_ospf);
-#endif
-#ifdef CONFIG_PIPE
-  proto_build(&proto_pipe);
-#endif
-#ifdef CONFIG_BGP
-  proto_build(&proto_bgp);
-#endif
-#ifdef CONFIG_BFD
-  proto_build(&proto_bfd);
-  bfd_init_all();
-#endif
-#ifdef CONFIG_BABEL
-  proto_build(&proto_babel);
-#endif
-#ifdef CONFIG_RPKI
-  proto_build(&proto_rpki);
-#endif
-#ifdef CONFIG_PERF
-  proto_build(&proto_perf);
-#endif
+  protos_build_gen();
 
   proto_pool = rp_new(&root_pool, "Protocols");
   proto_shutdown_timer = tm_new(proto_pool);
@@ -2240,8 +2212,13 @@ proto_apply_cmd_symbol(const struct symbol *s, void (* cmd)(struct proto *, uint
     return;
   }
 
-  cmd(s->proto->proto, arg, 0);
-  cli_msg(0, "");
+  if (s->proto->proto)
+  {
+    cmd(s->proto->proto, arg, 0);
+    cli_msg(0, "");
+  }
+  else
+    cli_msg(9002, "%s does not exist", s->name);
 }
 
 static void
