@@ -28,6 +28,7 @@
 
 #define EA_BABEL_METRIC		EA_CODE(PROTOCOL_BABEL, 0)
 #define EA_BABEL_ROUTER_ID	EA_CODE(PROTOCOL_BABEL, 1)
+#define EA_BABEL_SEQNO		EA_CODE(PROTOCOL_BABEL, 2)
 
 #define BABEL_MAGIC		42
 #define BABEL_VERSION		2
@@ -47,6 +48,8 @@
 #define BABEL_ROUTE_REFRESH_FACTOR(X)	((btime)(X)*5/2)	/* 2.5 */
 #define BABEL_SEQNO_REQUEST_RETRY	4
 #define BABEL_SEQNO_REQUEST_EXPIRY	(2 S_)
+#define BABEL_SEQNO_FORWARD_EXPIRY	(10 S_)
+#define BABEL_SEQNO_DUP_SUPPRESS_TIME	(1 S_)
 #define BABEL_GARBAGE_INTERVAL		(300 S_)
 #define BABEL_RXCOST_WIRED		96
 #define BABEL_RXCOST_WIRELESS		256
@@ -239,7 +242,6 @@ struct babel_neighbor {
   btime init_expiry;
 
   list routes;				/* Routes this neighbour has sent us (struct babel_route) */
-  list requests;			/* Seqno requests bound to this neighbor */
 };
 
 struct babel_source {
@@ -269,13 +271,13 @@ struct babel_route {
 
 struct babel_seqno_request {
   node n;
-  node nbr_node;
   u64 router_id;
   u16 seqno;
+  u8 forwarded;
   u8 hop_count;
   u8 count;
   btime expires;
-  struct babel_neighbor *nbr;
+  btime dup_suppress_time;
 };
 
 struct babel_entry {
