@@ -121,18 +121,16 @@ rpki_table_add_roa(struct rpki_cache *cache, struct channel *channel, const net_
   struct rpki_proto *p = cache->p;
 
   rta a0 = {
-    .src = p->p.main_source,
+    .pref = channel->preference,
     .source = RTS_RPKI,
     .scope = SCOPE_UNIVERSE,
     .dest = RTD_NONE,
   };
 
   rta *a = rta_lookup(&a0);
-  rte *e = rte_get_temp(a);
+  rte *e = rte_get_temp(a, p->p.main_source);
 
-  e->pflags = 0;
-
-  rte_update2(channel, &pfxr->n, e, a0.src);
+  rte_update2(channel, &pfxr->n, e, e->src);
 }
 
 void
@@ -278,12 +276,13 @@ rpki_cache_change_state(struct rpki_cache *cache, const enum rpki_cache_state ne
 
   case RPKI_CS_NO_INCR_UPDATE_AVAIL:
     /* Server was unable to answer the last Serial Query and sent Cache Reset. */
-    rpki_cache_change_state(cache, RPKI_CS_RESET);
-    break;
-
   case RPKI_CS_ERROR_NO_DATA_AVAIL:
     /* No validation records are available on the cache server. */
-    rpki_cache_change_state(cache, RPKI_CS_RESET);
+
+    if (old_state == RPKI_CS_ESTABLISHED)
+      rpki_cache_change_state(cache, RPKI_CS_RESET);
+    else
+      rpki_schedule_next_retry(cache);
     break;
 
   case RPKI_CS_ERROR_FATAL:
@@ -453,6 +452,11 @@ rpki_retry_hook(timer *tm)
     }
     break;
 
+  case RPKI_CS_NO_INCR_UPDATE_AVAIL:
+  case RPKI_CS_ERROR_NO_DATA_AVAIL:
+    rpki_cache_change_state(cache, RPKI_CS_RESET);
+    break;
+
   default:
     rpki_cache_change_state(cache, RPKI_CS_CONNECTING);
     break;
@@ -960,3 +964,9 @@ struct protocol proto_rpki = {
   .reconfigure = 	rpki_reconfigure,
   .get_status = 	rpki_get_status,
 };
+
+void
+rpki_build(void)
+{
+  proto_build(&proto_rpki);
+}
